@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-close-confirmation-dialog.c 5584 2007-03-24 13:41:51Z pborelli $
+ * $Id$
  */
 
 #ifdef HAVE_CONFIG_H
@@ -196,7 +196,8 @@ gedit_close_confirmation_dialog_init (GeditCloseConfirmationDialog *dlg)
 			& GEDIT_LOCKDOWN_SAVE_TO_DISK;
 
 	gtk_container_set_border_width (GTK_CONTAINER (dlg), 5);		
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dlg)->vbox), 14);
+	gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dlg))),
+			     14);
 	gtk_window_set_resizable (GTK_WINDOW (dlg), FALSE);
 	gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dlg), TRUE);
@@ -517,7 +518,7 @@ build_single_doc_dialog (GeditCloseConfirmationDialog *dlg)
 	}
 	else
 	{
-		str = g_markup_printf_escaped (_("Save the changes to document \"%s\" before closing?"),
+		str = g_markup_printf_escaped (_("Save changes to document \"%s\" before closing?"),
 					       doc_name);
 	}
 
@@ -553,7 +554,7 @@ build_single_doc_dialog (GeditCloseConfirmationDialog *dlg)
 		      
 	gtk_box_pack_start (GTK_BOX (vbox), secondary_label, FALSE, FALSE, 0);
 
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), 
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dlg))),
 			    hbox, 
 	                    FALSE, 
 			    FALSE, 
@@ -676,7 +677,7 @@ build_multiple_docs_dialog (GeditCloseConfirmationDialog *dlg)
 
 	hbox = gtk_hbox_new (FALSE, 12);
 	gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
-  	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), 
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dlg))),
 			    hbox, TRUE, TRUE, 0);
 
 	/* Image */
