@@ -139,12 +139,27 @@ gedit_document_saver_finalize (GObject *object)
 	G_OBJECT_CLASS (gedit_document_saver_parent_class)->finalize (object);
 }
 
+static void
+gedit_document_saver_dispose (GObject *object)
+{
+	GeditDocumentSaver *saver = GEDIT_DOCUMENT_SAVER (object);
+
+	if (saver->info != NULL)
+	{
+		g_object_unref (saver->info);
+		saver->info = NULL;
+	}
+
+	G_OBJECT_CLASS (gedit_document_saver_parent_class)->dispose (object);
+}
+
 static void 
 gedit_document_saver_class_init (GeditDocumentSaverClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize = gedit_document_saver_finalize;
+	object_class->dispose = gedit_document_saver_dispose;
 	object_class->set_property = gedit_document_saver_set_property;
 	object_class->get_property = gedit_document_saver_get_property;
 
@@ -336,54 +351,69 @@ gedit_document_saver_write_document_contents (GeditDocumentSaver  *saver,
 
 	/* Truncate the file to 0, in case it was not empty */
 	if (res)
+	{
 		res = (ftruncate (fd, 0) == 0);
+	}
 
 	/* Save the file content */
-	if (res)
+	if (len > 0)
 	{
-		const gchar *write_buffer = contents;
-		gssize to_write = len;
-
-		do
+		if (res)
 		{
-			written = write (fd, write_buffer, to_write);
-			if (written == -1)
+			const gchar *write_buffer = contents;
+			gssize to_write = len;
+
+			do
 			{
-				if (errno == EINTR)
-					continue;
+				written = write (fd, write_buffer, to_write);
+				if (written == -1)
+				{
+					if (errno == EINTR)
+						continue;
 
-				res = FALSE;
+					res = FALSE;
 
-				break;
+					break;
+				}
+
+				to_write -= written;
+				write_buffer += written;
+			}
+			while (to_write > 0);
+		}
+
+		/* make sure files are always terminated with \n (see bug #95676). Note
+		   that we strip the trailing \n when loading the file */
+		if (res)
+		{
+			gchar *n_buf;
+			gsize n_len;
+
+			n_buf = gedit_document_saver_get_end_newline (saver, &n_len);
+			if (n_buf != NULL)
+			{
+				written = write (fd, n_buf, n_len);
+				res = (written != -1 && (gsize) written == n_len);
+				g_free (n_buf);
+			}
+			else
+			{
+				g_warning ("Cannot add '\\n' at the end of the file.");
 			}
-
-			to_write -= written;
-			write_buffer += written;
 		}
-		while (to_write > 0);
 	}
 
-	/* make sure files are always terminated with \n (see bug #95676). Note
-	   that we strip the trailing \n when loading the file */
-	if (res)
+#ifdef HAVE_FSYNC
+	/* Ensure that all the data reaches disk */
+	if (res && fsync (fd) != 0)
 	{
-		gchar *n_buf;
-		gsize n_len;
-
-		n_buf = gedit_document_saver_get_end_newline (saver, &n_len);
-		if (n_buf != NULL)
-		{
-			written = write (fd, n_buf, n_len);
-			res = (written != -1 && (gsize) written == n_len);
-			g_free (n_buf);
-		}
-		else
-		{
-			g_warning ("Cannot add '\\n' at the end of the file.");
-		}
+		g_set_error (error,
+			     G_IO_ERROR,
+			     g_io_error_from_errno (errno),
+			     "%s", g_strerror (errno));
+		res = FALSE;
 	}
-
-	g_free (contents);
+#endif
 
 	if (!res)
 	{
@@ -393,6 +423,8 @@ gedit_document_saver_write_document_contents (GeditDocumentSaver  *saver,
 			     "%s", g_strerror (errno));
 	}
 
+	g_free (contents);
+
 	return res;
 }
 
@@ -424,7 +456,7 @@ gedit_document_saver_saving (GeditDocumentSaver *saver,
 
 void
 gedit_document_saver_save (GeditDocumentSaver     *saver,
-			   time_t                  old_mtime)
+			   GTimeVal               *old_mtime)
 {
 	gedit_debug (DEBUG_SAVER);
 
@@ -454,6 +486,14 @@ gedit_document_saver_save (GeditDocumentSaver     *saver,
 	GEDIT_DOCUMENT_SAVER_GET_CLASS (saver)->save (saver, old_mtime);
 }
 
+GeditDocument *
+gedit_document_saver_get_document (GeditDocumentSaver *saver)
+{
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT_SAVER (saver), NULL);
+
+	return saver->document;
+}
+
 const gchar *
 gedit_document_saver_get_uri (GeditDocumentSaver *saver)
 {
@@ -462,22 +502,6 @@ gedit_document_saver_get_uri (GeditDocumentSaver *saver)
 	return saver->uri;
 }
 
-const gchar *
-gedit_document_saver_get_content_type (GeditDocumentSaver *saver)
-{
-	g_return_val_if_fail (GEDIT_IS_DOCUMENT_SAVER (saver), NULL);
-
-	return GEDIT_DOCUMENT_SAVER_GET_CLASS (saver)->get_content_type (saver);
-}
-
-time_t
-gedit_document_saver_get_mtime (GeditDocumentSaver *saver)
-{
-	g_return_val_if_fail (GEDIT_IS_DOCUMENT_SAVER (saver), 0);
-
-	return GEDIT_DOCUMENT_SAVER_GET_CLASS (saver)->get_mtime (saver);
-}
-
 /* Returns 0 if file size is unknown */
 goffset
 gedit_document_saver_get_file_size (GeditDocumentSaver *saver)
@@ -494,3 +518,11 @@ gedit_document_saver_get_bytes_written (GeditDocumentSaver *saver)
 
 	return GEDIT_DOCUMENT_SAVER_GET_CLASS (saver)->get_bytes_written (saver);
 }
+
+GFileInfo *
+gedit_document_saver_get_info (GeditDocumentSaver *saver)
+{
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT_SAVER (saver), NULL);
+
+	return saver->info;
+}
