@@ -44,7 +44,6 @@
 #include "gedit-document.h"
 #include "gedit-debug.h"
 #include "gedit-utils.h"
-#include "gedit-metadata-manager.h"
 #include "gedit-language-manager.h"
 #include "gedit-style-scheme-manager.h"
 #include "gedit-document-loader.h"
@@ -53,6 +52,12 @@
 #include "gedit-enum-types.h"
 #include "gedittextregion.h"
 
+#ifdef G_OS_WIN32
+#include "gedit-metadata-manager.h"
+#else
+#define METADATA_QUERY "metadata::*"
+#endif
+
 #undef ENABLE_PROFILE 
 
 #ifdef ENABLE_PROFILE
@@ -98,21 +103,16 @@ static void	delete_range_cb 		(GeditDocument *doc,
 			     
 struct _GeditDocumentPrivate
 {
-	gint	     readonly : 1;
-	gint	     last_save_was_manually : 1; 	
-	gint	     language_set_by_user : 1;
-	gint         stop_cursor_moved_emission : 1;
-	gint         dispose_has_run : 1;
-
 	gchar	    *uri;
 	gint 	     untitled_number;
 
+	GFileInfo   *metadata_info;
+
 	const GeditEncoding *encoding;
 
 	gchar	    *content_type;
 
-	time_t       mtime;
-
+	GTimeVal     mtime;
 	GTimeVal     time_of_last_save_or_load;
 
 	guint        search_flags;
@@ -129,13 +129,19 @@ struct _GeditDocumentPrivate
 	/* Saving stuff */
 	GeditDocumentSaver *saver;
 
-	/* Search highlighting support variables */	
+	/* Search highlighting support variables */
 	GeditTextRegion *to_search_region;
 	GtkTextTag      *found_tag;
-	
+
 	/* Mount operation factory */
 	GeditMountOperationFactory  mount_operation_factory;
 	gpointer		    mount_operation_userdata;
+
+	gint readonly : 1;
+	gint last_save_was_manually : 1; 
+	gint language_set_by_user : 1;
+	gint stop_cursor_moved_emission : 1;
+	gint dispose_has_run : 1;
 };
 
 enum {
@@ -228,19 +234,7 @@ gedit_document_dispose (GObject *object)
 	{
 		GtkTextIter iter;
 		gchar *position;
-
-		gtk_text_buffer_get_iter_at_mark (
-				GTK_TEXT_BUFFER (doc),			
-				&iter,
-				gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (doc)));
-
-		position = g_strdup_printf ("%d", 
-					    gtk_text_iter_get_offset (&iter));
-
-		gedit_metadata_manager_set (doc->priv->uri,
-					    "position",
-					    position);
-		g_free (position);
+		const gchar *language = NULL;
 
 		if (doc->priv->language_set_by_user)
 		{
@@ -248,10 +242,28 @@ gedit_document_dispose (GObject *object)
 
 			lang = gedit_document_get_language (doc);
 
-			gedit_metadata_manager_set (doc->priv->uri,
-				    "language",
-				    (lang == NULL) ? "_NORMAL_" : gtk_source_language_get_id (lang));
+			if (lang == NULL)
+				language = "_NORMAL_";
+			else
+				language = gtk_source_language_get_id (lang);
 		}
+
+		gtk_text_buffer_get_iter_at_mark (
+				GTK_TEXT_BUFFER (doc),
+				&iter,
+				gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (doc)));
+
+		position = g_strdup_printf ("%d", 
+					    gtk_text_iter_get_offset (&iter));
+
+		if (language == NULL)
+			gedit_document_set_metadata (doc, GEDIT_METADATA_ATTRIBUTE_POSITION,
+						     position, NULL);
+		else
+			gedit_document_set_metadata (doc, GEDIT_METADATA_ATTRIBUTE_POSITION,
+						     position, GEDIT_METADATA_ATTRIBUTE_LANGUAGE,
+						     language, NULL);
+		g_free (position);
 	}
 
 	if (doc->priv->loader)
@@ -260,6 +272,12 @@ gedit_document_dispose (GObject *object)
 		doc->priv->loader = NULL;
 	}
 
+	if (doc->priv->metadata_info != NULL)
+	{
+		g_object_unref (doc->priv->metadata_info);
+		doc->priv->metadata_info = NULL;
+	}
+
 	doc->priv->dispose_has_run = TRUE;
 
 	G_OBJECT_CLASS (gedit_document_parent_class)->dispose (object);
@@ -625,9 +643,9 @@ set_language (GeditDocument     *doc,
 
 	if (set_by_user && (doc->priv->uri != NULL))
 	{
-		gedit_metadata_manager_set (doc->priv->uri,
-			    "language",
-			    (lang == NULL) ? "_NORMAL_" : gtk_source_language_get_id (lang));
+		gedit_document_set_metadata (doc, GEDIT_METADATA_ATTRIBUTE_LANGUAGE,
+			(lang == NULL) ? "_NORMAL_" : gtk_source_language_get_id (lang),
+			NULL);
 	}
 
 	doc->priv->language_set_by_user = set_by_user;
@@ -653,9 +671,8 @@ set_encoding (GeditDocument       *doc,
 
 		charset = gedit_encoding_get_charset (encoding);
 
-		gedit_metadata_manager_set (doc->priv->uri,
-					    "encoding",
-					    charset);
+		gedit_document_set_metadata (doc, GEDIT_METADATA_ATTRIBUTE_ENCODING,
+					     charset, NULL);
 	}
 
 	g_object_notify (G_OBJECT (doc), "encoding");
@@ -689,6 +706,72 @@ get_default_style_scheme (void)
 	return def_style;
 }
 
+static GtkSourceLanguage *
+guess_language (GeditDocument *doc,
+		const gchar   *content_type)
+{
+	gchar *data;
+	GtkSourceLanguage *language = NULL;
+
+	data = gedit_document_get_metadata (doc, GEDIT_METADATA_ATTRIBUTE_LANGUAGE);
+
+	if (data != NULL)
+	{
+		gedit_debug_message (DEBUG_DOCUMENT, "Language from metadata: %s", data);
+
+		if (strcmp (data, "_NORMAL_") != 0)
+		{
+			language = gtk_source_language_manager_get_language (
+						gedit_get_language_manager (),
+						data);
+		}
+
+		g_free (data);
+	}
+	else
+	{
+		GFile *file;
+		gchar *basename;
+
+		file = gedit_document_get_location (doc);
+
+		if (file)
+		{
+			gedit_debug_message (DEBUG_DOCUMENT, "Sniffing Language");
+
+			basename = g_file_get_basename (file);
+
+			language = gtk_source_language_manager_guess_language (
+						gedit_get_language_manager (),
+						basename,
+						content_type);
+
+			g_free (basename);
+			g_object_unref (file);
+		}
+	}
+
+	return language;
+}
+
+static void
+on_content_type_changed (GeditDocument *doc,
+			 GParamSpec    *pspec,
+			 gpointer       useless)
+{
+	if (!doc->priv->language_set_by_user)
+	{
+		GtkSourceLanguage *language;
+
+		language = guess_language (doc, doc->priv->content_type);
+
+		gedit_debug_message (DEBUG_DOCUMENT, "Language: %s",
+				     language != NULL ? gtk_source_language_get_name (language) : "None");
+
+		set_language (doc, language, FALSE);
+	}
+}
+
 static void
 gedit_document_init (GeditDocument *doc)
 {
@@ -701,6 +784,8 @@ gedit_document_init (GeditDocument *doc)
 	doc->priv->uri = NULL;
 	doc->priv->untitled_number = get_untitled_number ();
 
+	doc->priv->metadata_info = NULL;
+
 	doc->priv->content_type = g_content_type_from_mime_type ("text/plain");
 
 	doc->priv->readonly = FALSE;
@@ -712,7 +797,8 @@ gedit_document_init (GeditDocument *doc)
 
 	doc->priv->dispose_has_run = FALSE;
 
-	doc->priv->mtime = 0;
+	doc->priv->mtime.tv_sec = 0;
+	doc->priv->mtime.tv_usec = 0;
 
 	g_get_current_time (&doc->priv->time_of_last_save_or_load);
 
@@ -740,7 +826,12 @@ gedit_document_init (GeditDocument *doc)
 	g_signal_connect_after (doc, 
 			  	"delete-range",
 			  	G_CALLBACK (delete_range_cb),
-			  	NULL);		
+			  	NULL);
+
+	g_signal_connect (doc,
+			  "notify::content-type",
+			  G_CALLBACK (on_content_type_changed),
+			  NULL);
 }
 
 GeditDocument *
@@ -751,57 +842,77 @@ gedit_document_new (void)
 	return GEDIT_DOCUMENT (g_object_new (GEDIT_TYPE_DOCUMENT, NULL));
 }
 
-static GtkSourceLanguage *
-guess_language (const gchar *uri,
-		const gchar *content_type)
-
+static void
+set_content_type (GeditDocument *doc,
+		  const gchar   *content_type)
 {
-	gchar *data;
-	GtkSourceLanguage *language = NULL;
+	gedit_debug (DEBUG_DOCUMENT);
 
-	data = gedit_metadata_manager_get (uri, "language");
+	g_free (doc->priv->content_type);
 
-	if (data != NULL)
+	if (content_type != NULL)
 	{
-		gedit_debug_message (DEBUG_DOCUMENT, "Language from metadata: %s", data);
-
-		if (strcmp (data, "_NORMAL_") != 0)
-		{
-			language = gtk_source_language_manager_get_language (
-						gedit_get_language_manager (),
-						data);
-		}
-
-		g_free (data);
+		doc->priv->content_type = g_strdup (content_type);
 	}
 	else
 	{
 		GFile *file;
-		gchar *basename;
+		gchar *guessed_type = NULL;
 
-		gedit_debug_message (DEBUG_DOCUMENT, "Sniffing Language");
+		/* If content type is null, we guess from the filename */
+		file = gedit_document_get_location (doc);
+		if (file != NULL)
+		{
+			gchar *basename;
 
-		file = g_file_new_for_uri (uri);
-		basename = g_file_get_basename (file);
+			basename = g_file_get_basename (file);
+			guessed_type = g_content_type_guess (basename, NULL, 0, NULL);
 
-		language = gtk_source_language_manager_guess_language (
-					gedit_get_language_manager (),
-					basename,
-					content_type);
+			g_free (basename);
+			g_object_unref (file);
+		}
 
-		g_free (basename);
-		g_object_unref (file);
+		if (guessed_type == NULL || g_content_type_is_unknown (guessed_type))
+		{
+			g_free (guessed_type);
+			guessed_type = g_content_type_from_mime_type ("text/plain");
+		}
+
+		doc->priv->content_type = guessed_type;
 	}
 
-	return language;
+	g_object_notify (G_OBJECT (doc), "content-type");
 }
 
-/* If content type is null, we guess from the filename */
-/* If uri is null, we only set the content-type */
+#ifndef G_OS_WIN32
+static void
+query_info_cb (GFile         *source,
+	       GAsyncResult  *res,
+	       GeditDocument *doc)
+{
+	GError *error = NULL;
+
+	doc->priv->metadata_info = g_file_query_info_finish (source,
+							     res,
+							     &error);
+
+	if (error != NULL)
+	{
+		if (error->code != G_FILE_ERROR_ISDIR)
+			g_warning ("%s", error->message);
+		g_error_free (error);
+		
+		return;
+	}
+
+	
+	on_content_type_changed (doc, NULL, NULL);
+}
+#endif
+
 static void
 set_uri (GeditDocument *doc,
-	 const gchar   *uri,
-	 const gchar   *content_type)
+	 const gchar   *uri)
 {
 	gedit_debug (DEBUG_DOCUMENT);
 
@@ -822,35 +933,24 @@ set_uri (GeditDocument *doc,
 		}
 	}
 
-	g_free (doc->priv->content_type);
+#ifndef G_OS_WIN32
+	GFile *location;
 
-	if (content_type != NULL)
+	/* Get the GFileInfo async so we can set the language from the metadata */
+	location = gedit_document_get_location (doc);
+	
+	if (location != NULL)
 	{
-		doc->priv->content_type = g_strdup (content_type);
-	}
-	else
-	{
-		if (doc->priv->uri != NULL)
-		{
-			doc->priv->content_type = g_content_type_guess (doc->priv->uri, NULL, 0, NULL);
-		}
-		else
-		{
-			doc->priv->content_type = g_content_type_from_mime_type ("text/plain");
-		}
-	}
-
-	if (!doc->priv->language_set_by_user)
-	{
-		GtkSourceLanguage *language;
-
-		language = guess_language (doc->priv->uri, doc->priv->content_type);
-
-		gedit_debug_message (DEBUG_DOCUMENT, "Language: %s",
-				     language != NULL ? gtk_source_language_get_name (language) : "None");
-
-		set_language (doc, language, FALSE);
+		g_file_query_info_async (location,
+					 METADATA_QUERY,
+					 G_FILE_QUERY_INFO_NONE,
+					 G_PRIORITY_DEFAULT,
+					 NULL,
+					 (GAsyncReadyCallback) query_info_cb,
+					 doc);
+		g_object_unref (location);
 	}
+#endif
 
 	g_object_notify (G_OBJECT (doc), "uri");
 	g_object_notify (G_OBJECT (doc), "shortname");
@@ -876,7 +976,8 @@ void
 gedit_document_set_uri (GeditDocument *doc,
 			const gchar   *uri)
 {
-	set_uri (doc, uri, doc->priv->content_type);
+	set_uri (doc, uri);
+	set_content_type (doc, NULL);
 }
 
 /* Never returns NULL */
@@ -996,7 +1097,9 @@ _gedit_document_check_externally_modified (GeditDocument *doc)
 	g_file_info_get_modification_time (info, &timeval);
 	g_object_unref (info);
 	
-	return timeval.tv_sec > doc->priv->mtime;
+	return (timeval.tv_sec > doc->priv->mtime.tv_sec) ||
+	       (timeval.tv_sec == doc->priv->mtime.tv_sec && 
+	       timeval.tv_usec > doc->priv->mtime.tv_usec);
 }
 
 static void
@@ -1019,23 +1122,38 @@ document_loader_loaded (GeditDocumentLoader *loader,
 	if (error == NULL)
 	{
 		GtkTextIter iter;
-		const gchar *content_type;
+		GFileInfo *info;
+		const gchar *content_type = NULL;
+		gboolean read_only = FALSE;
+		GTimeVal mtime = {0, 0};
 
-		content_type = gedit_document_loader_get_content_type (loader);
+		info = gedit_document_loader_get_info (loader);
 
-		doc->priv->mtime = gedit_document_loader_get_mtime (loader);
+		if (info)
+		{
+			if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE))
+				content_type = g_file_info_get_attribute_string (info,
+										 G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE);
+
+			if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_TIME_MODIFIED))
+				g_file_info_get_modification_time (info, &mtime);
+
+			if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))
+				read_only = !g_file_info_get_attribute_boolean (info,
+										G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE);
+		}
+
+		doc->priv->mtime = mtime;
+
+		set_readonly (doc, read_only);
 
 		g_get_current_time (&doc->priv->time_of_last_save_or_load);
 
-		set_readonly (doc,
-		      gedit_document_loader_get_readonly (loader));
-
 		set_encoding (doc, 
 			      gedit_document_loader_get_encoding (loader),
 			      (doc->priv->requested_encoding != NULL));
-		      
-		/* We already set the uri */
-		set_uri (doc, NULL, content_type);
+		
+		set_content_type (doc, content_type);
 
 		/* move the cursor at the requested line if any */
 		if (doc->priv->requested_line_pos > 0)
@@ -1051,8 +1169,7 @@ document_loader_loaded (GeditDocumentLoader *loader,
 			gchar *pos;
 			gint offset;
 
-			pos = gedit_metadata_manager_get (doc->priv->uri,
-							  "position");
+			pos = gedit_document_get_metadata (doc, GEDIT_METADATA_ATTRIBUTE_POSITION);
 
 			offset = pos ? atoi (pos) : 0;
 			g_free (pos);
@@ -1115,10 +1232,16 @@ document_loader_loading (GeditDocumentLoader *loader,
 	}
 	else
 	{
-		goffset size;
+		goffset size = 0;
 		goffset read;
+		GFileInfo *info;
+
+		info = gedit_document_loader_get_info (loader);
+
+		if (info && g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_STANDARD_SIZE))
+			size = g_file_info_get_attribute_uint64 (info,
+								 G_FILE_ATTRIBUTE_STANDARD_SIZE);
 
-		size = gedit_document_loader_get_file_size (loader);
 		read = gedit_document_loader_get_bytes_read (loader);
 
 		g_signal_emit (doc, 
@@ -1152,7 +1275,8 @@ gedit_document_load_real (GeditDocument       *doc,
 	doc->priv->requested_encoding = encoding;
 	doc->priv->requested_line_pos = line_pos;
 
-	set_uri (doc, uri, NULL);
+	set_uri (doc, uri);
+	set_content_type (doc, NULL);
 
 	gedit_document_loader_load (doc->priv->loader);
 }
@@ -1212,13 +1336,27 @@ document_saver_saving (GeditDocumentSaver *saver,
 		if (error == NULL)
 		{
 			const gchar *uri;
-			const gchar *content_type;
+			const gchar *content_type = NULL;
+			GTimeVal mtime = {0, 0};
+			GFileInfo *info;
 
 			uri = gedit_document_saver_get_uri (saver);
+			set_uri (doc, uri);
 
-			content_type = gedit_document_saver_get_content_type (saver);
+			info = gedit_document_saver_get_info (saver);
 
-			doc->priv->mtime = gedit_document_saver_get_mtime (saver);
+			if (info != NULL)
+			{
+				if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE))
+					content_type = g_file_info_get_attribute_string (info,
+											 G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE);
+
+				if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_TIME_MODIFIED))
+					g_file_info_get_modification_time (info, &mtime);
+			}
+
+			set_content_type (doc, content_type);
+			doc->priv->mtime = mtime;
 
 			g_get_current_time (&doc->priv->time_of_last_save_or_load);
 
@@ -1227,8 +1365,6 @@ document_saver_saving (GeditDocumentSaver *saver,
 			gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (doc),
 						      FALSE);
 
-			set_uri (doc, uri, content_type);
-
 			set_encoding (doc, 
 				      doc->priv->requested_encoding, 
 				      TRUE);
@@ -1280,7 +1416,7 @@ gedit_document_save_real (GeditDocument          *doc,
 	doc->priv->requested_encoding = encoding;
 
 	gedit_document_saver_save (doc->priv->saver,
-				   doc->priv->mtime);
+				   &doc->priv->mtime);
 }
 
 /**
@@ -2285,3 +2421,155 @@ _gedit_document_create_mount_operation (GeditDocument *doc)
 		return doc->priv->mount_operation_factory (doc, 
 						           doc->priv->mount_operation_userdata);
 }
+
+#ifdef G_OS_WIN32
+gchar *
+gedit_document_get_metadata (GeditDocument *doc,
+			     const gchar   *key)
+{
+	gchar *value = NULL;
+
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), NULL);
+	g_return_val_if_fail (key != NULL, NULL);
+
+	if (doc->priv->uri != NULL)
+		value = gedit_metadata_manager_get (doc->priv->uri, key);
+
+	return value;
+}
+
+void
+gedit_document_set_metadata (GeditDocument *doc,
+			     const gchar   *first_key,
+			     ...)
+{
+	const gchar *key;
+	const gchar *value;
+	va_list var_args;
+
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+	g_return_if_fail (first_key != NULL);
+
+	va_start (var_args, first_key);
+
+	for (key = first_key; key; key = va_arg (var_args, const gchar *))
+	{
+		value = va_arg (var_args, const gchar *);
+		
+		gedit_metadata_manager_set (doc->priv->uri,
+					    key,
+					    value);
+	}
+
+	va_end (var_args);
+}
+
+#else
+
+/**
+ * gedit_document_get_metadata:
+ * @doc: a #GeditDocument
+ * @key: name of the key
+ *
+ * Gets the metadata assigned to @key.
+ *
+ * Returns: the value assigned to @key.
+ */
+gchar *
+gedit_document_get_metadata (GeditDocument *doc,
+			     const gchar   *key)
+{
+	gchar *value = NULL;
+
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), NULL);
+	g_return_val_if_fail (key != NULL, NULL);
+
+	if (doc->priv->metadata_info && g_file_info_has_attribute (doc->priv->metadata_info,
+								   key))
+	{
+		value = g_strdup (g_file_info_get_attribute_string (doc->priv->metadata_info,
+								    key));
+	}
+
+	return value;
+}
+
+static void
+set_attributes_cb (GObject      *source,
+		   GAsyncResult *res,
+		   gpointer      useless)
+{
+	g_file_set_attributes_finish (G_FILE (source),
+				      res,
+				      NULL,
+				      NULL);
+}
+
+/**
+ * gedit_document_set_metadata:
+ * @doc: a #GeditDocument
+ * @first_key: name of the first key to set
+ * @...: value for the first key, followed optionally by more key/value pairs,
+ * followed by %NULL.
+ *
+ * Sets metadata on a document.
+ */
+void
+gedit_document_set_metadata (GeditDocument *doc,
+			     const gchar   *first_key,
+			     ...)
+{
+	const gchar *key;
+	const gchar *value;
+	va_list var_args;
+	GFileInfo *info;
+	GFile *location;
+
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+	g_return_if_fail (first_key != NULL);
+
+	info = g_file_info_new ();
+
+	va_start (var_args, first_key);
+
+	for (key = first_key; key; key = va_arg (var_args, const gchar *))
+	{
+		value = va_arg (var_args, const gchar *);
+		
+		if (value != NULL)
+		{
+			g_file_info_set_attribute_string (info,
+							  key, value);
+		}
+		else
+		{
+			/* Unset the key */
+			g_file_info_set_attribute (info, key,
+						   G_FILE_ATTRIBUTE_TYPE_INVALID,
+						   NULL);
+		}
+	}
+
+	va_end (var_args);
+
+	if (doc->priv->metadata_info != NULL)
+		g_file_info_copy_into (info, doc->priv->metadata_info);
+
+	location = gedit_document_get_location (doc);
+
+	if (location != NULL)
+	{
+		g_file_set_attributes_async (location,
+					     info,
+					     G_FILE_QUERY_INFO_NONE,
+					     G_PRIORITY_DEFAULT,
+					     NULL,
+					     set_attributes_cb,
+					     NULL);
+
+		g_object_unref (location);
+	}
+	
+	g_object_unref (info);
+}
+#endif
