@@ -22,11 +22,18 @@ import gtk
 from gtk import gdk
 import gio
 import gedit
+import gtksourceview2 as gsv
+import gobject
 
 from Library import Library
 from Snippet import Snippet
 from Placeholder import *
-from SnippetComplete import SnippetComplete
+import Completion
+
+class DynamicSnippet(dict):
+        def __init__(self, text):
+                self['text'] = text
+                self.valid = True
 
 class Document:
         TAB_KEY_VAL = (gtk.keysyms.Tab, \
@@ -48,6 +55,9 @@ class Document:
                 self.language_id = 0
                 self.timeout_update_id = 0
                 
+                self.provider = Completion.Provider(_('Snippets'), self.language_id, self.on_proposal_activated)
+                self.defaults_provider = Completion.Defaults(self.on_default_activated)
+                
                 # Always have a reference to the global snippets
                 Library().ref(None)
                 self.set_view(view)
@@ -88,12 +98,13 @@ class Document:
         def _set_view(self, view):
                 if self.view:
                         buf = self.view.get_buffer()
-                        
+
                         # Remove signals
                         signals = {self.view: ('key-press-event', 'destroy', 
-                                               'notify::editable', 'drag-data-received'),
-                                   buf:       ('notify::language', 'changed', 'cursor-moved', 'insert-text')}
-                        
+                                               'notify::editable', 'drag-data-received', 'expose-event'),
+                                   buf:       ('notify::language', 'changed', 'cursor-moved', 'insert-text'),
+                                   self.view.get_completion(): ('hide',)}
+
                         for obj, sig in signals.items():
                                 for s in sig:
                                         self.disconnect_signal(obj, s)
@@ -101,6 +112,10 @@ class Document:
                         # Remove all active snippets
                         for snippet in list(self.active_snippets):
                                 self.deactivate_snippet(snippet, True)
+                        
+                        completion = self.view.get_completion()
+                        completion.remove_provider(self.provider)
+                        completion.remove_provider(self.defaults_provider)                        
 
                 self.view = view
                 
@@ -115,12 +130,21 @@ class Document:
                         self.connect_signal(buf, 'notify::language', self.on_notify_language)
                         self.connect_signal(view, 'notify::editable', self.on_notify_editable)
                         self.connect_signal(view, 'drag-data-received', self.on_drag_data_received)
+                        self.connect_signal_after(view, 'expose-event', self.on_expose_event)
 
                         self.update_language()
+
+                        completion = view.get_completion()
+                        completion.add_provider(self.provider)
+                        
+                        completion.add_provider(self.defaults_provider)
+                        
+                        self.connect_signal(completion, 'hide', self.on_completion_hide)
                 elif self.language_id != 0:
                         langid = self.language_id
                         
                         self.language_id = None;
+                        self.provider.language_id = self.language_id
                         
                         if self.instance:
                                 self.instance.language_changed(self)
@@ -157,6 +181,7 @@ class Document:
                         Library().unref(langid)
 
                 Library().ref(self.language_id)
+                self.provider.language_id = self.language_id
 
         def accelerator_activate(self, keyval, mod):
                 if not self.view or not self.view.get_editable():
@@ -174,7 +199,8 @@ class Document:
                         self.apply_snippet(snippets[0])
                 else:
                         # Do the fancy completion dialog
-                        return self.show_completion(snippets)
+                        self.provider.set_proposals(snippets)
+                        self.view.show_completion((self,))
 
                 return True
 
@@ -295,6 +321,7 @@ class Document:
 
                 if current:
                         # Signal this placeholder to end action
+                        self.view.get_completion().hide()
                         current.leave()
                         
                         if current.__class__ == PlaceholderEnd:
@@ -307,6 +334,11 @@ class Document:
                         
                         if next.__class__ == PlaceholderEnd:
                                 last = next
+                        elif len(next.defaults) > 1 and next.get_text() == next.default:
+                                self.defaults_provider.set_defaults(next.defaults)
+                                
+                                cm = self.view.get_completion()
+                                cm.show([self.defaults_provider], cm.create_context())
 
                 if last:
                         # This is the end of the placeholder, remove the snippet etc
@@ -399,20 +431,30 @@ class Document:
                         return ''
 
         def env_get_documents_uri(self, buf):
-                documents_uri = [doc.get_uri()
-                                 for doc in self.view.get_toplevel().get_documents()
-                                 if doc.get_uri() is not None]
+                toplevel = self.view.get_toplevel()
+                
+                if isinstance(toplevel, gedit.Window):
+                        documents_uri = [doc.get_uri()
+                                         for doc in toplevel.get_documents()
+                                         if doc.get_uri() is not None]
+                else:
+                        documents_uri = []
                 
                 return ' '.join(documents_uri)
 
         def env_get_documents_path(self, buf):
-                documents_uri = [doc.get_uri()
-                                 for doc in self.view.get_toplevel().get_documents()
-                                 if doc.get_uri() is not None]
+                toplevel = self.view.get_toplevel()
+                
+                if isinstance(toplevel, gedit.Window):
+                        documents_uri = [doc.get_uri()
+                                         for doc in toplevel.get_documents()
+                                         if doc.get_uri() is not None]
 
-                documents_path = [gio.File(uri).get_path()
-                                 for uri in documents_uri
-                                 if gedit.utils.uri_has_file_scheme(uri)]
+                        documents_path = [gio.File(uri).get_path()
+                                         for uri in documents_uri
+                                         if gedit.utils.uri_has_file_scheme(uri)]
+                else:
+                        documents_path = []
                 
                 return ' '.join(documents_path)
 
@@ -511,13 +553,28 @@ class Document:
                 else:
                         self.goto_placeholder(self.active_placeholder, sn.placeholders[keys[0]])
 
+                if sn in self.active_snippets:
+                        # Check if we can get end_iter in view without moving the
+                        # current cursor position out of view
+                        cur = buf.get_iter_at_mark(buf.get_insert())
+                        last = sn.end_iter()
+
+                        curloc = self.view.get_iter_location(cur)
+                        lastloc = self.view.get_iter_location(last)
+
+                        if (lastloc.y + lastloc.height) - curloc.y <= \
+                           self.view.get_visible_rect().height:
+                                self.view.scroll_mark_onscreen(sn.end_mark)
+
                 buf.end_user_action()
                 self.view.grab_focus()
 
                 return True
 
-        def get_tab_tag(self, buf):
-                end = buf.get_iter_at_mark(buf.get_insert())
+        def get_tab_tag(self, buf, end = None):
+                if not end:
+                        end = buf.get_iter_at_mark(buf.get_insert())
+
                 start = end.copy()
                 
                 word = None
@@ -542,36 +599,54 @@ class Document:
                                         return (None, None, None)
                         else:
                                 return (None, None, None)
-                
+
                 return (word, start, end)
 
-        def run_snippet(self):        
+        def parse_and_run_snippet(self, data, iter):
+                self.apply_snippet(DynamicSnippet(data), iter, iter)
+
+        def run_snippet_trigger(self, trigger, bounds):
                 if not self.view:
                         return False
-                
+
+                snippets = Library().from_tag(trigger, self.language_id)
                 buf = self.view.get_buffer()
-                
-                # get the word preceding the current insertion position
-                (word, start, end) = self.get_tab_tag(buf)
-                
-                if not word:
-                        return self.skip_to_next_placeholder()
-                
-                snippets = Library().from_tag(word, self.language_id)
-                
+
                 if snippets:
                         if len(snippets) == 1:
-                                return self.apply_snippet(snippets[0], start, end)
+                                return self.apply_snippet(snippets[0], bounds[0], bounds[1])
                         else:
                                 # Do the fancy completion dialog
-                                return self.show_completion(snippets)
+                                self.provider.set_proposals(snippets)
+                                cm = self.view.get_completion()
+                                
+                                cm.show([self.provider], cm.create_context())
+                                return True
+
+                return False
+                
+        def run_snippet(self):
+                if not self.view:
+                        return False
+
+                buf = self.view.get_buffer()
+
+                # get the word preceding the current insertion position
+                (word, start, end) = self.get_tab_tag(buf)
+
+                if not word:
+                        return self.skip_to_next_placeholder()
+
+                if not self.run_snippet_trigger(word, (start, end)):
+                        return self.skip_to_next_placeholder()
+                else:
+                        return True
 
-                return self.skip_to_next_placeholder()
-        
         def deactivate_snippet(self, snippet, force = False):
                 buf = self.view.get_buffer()
                 remove = []
-                
+                ordered_remove = []
+
                 for tabstop in snippet.placeholders:
                         if tabstop == -1:
                                 placeholders = snippet.placeholders[-1]
@@ -586,8 +661,10 @@ class Document:
                                                 self.update_placeholders.remove(placeholder)
                                         elif placeholder in self.jump_placeholders:
                                                 placeholder[0].leave()
-                                                
+
                                         remove.append(placeholder)
+                                elif placeholder in self.ordered_placeholders:
+                                        ordered_remove.append(placeholder)
 
                 for placeholder in remove:
                         if placeholder == self.active_placeholder:
@@ -598,105 +675,30 @@ class Document:
 
                         placeholder.remove(force)
 
+                for placeholder in ordered_remove:
+                        self.ordered_placeholders.remove(placeholder)
+                        placeholder.remove(force)
+
                 snippet.deactivate()
                 self.active_snippets.remove(snippet)
-                
+
                 if len(self.active_snippets) == 0:
                         self.last_snippet_removed()
 
-        # Moves the completion window to a suitable place honoring the hint given
-        # by x and y. It tries to position the window so it's always visible on the
-        # screen.
-        def move_completion_window(self, complete, x, y):
-                MARGIN = 15
-                screen = self.view.get_screen()
-                
-                width = screen.get_width()
-                height = screen.get_height()
-                
-                cw, ch = complete.get_size()
-                
-                if x + cw > width:
-                        x = width - cw - MARGIN
-                elif x < MARGIN:
-                        x = MARGIN
-                
-                if y + ch > height:
-                        y = height - ch - MARGIN
-                elif y < MARGIN:
-                        y = MARGIN
-
-                complete.move(x, y)
-
-        # Show completion, shows a completion dialog in the view.
-        # If preset is not None then a completion dialog is shown with the snippets
-        # in the preset list. Otherwise it will try to find the word preceding the
-        # current cursor position. If such a word is found, it is taken as a 
-        # tab trigger prefix so that only snippets with a tab trigger prefixed with
-        # the word are in the list. If no such word can be found than all snippets
-        # are shown.
-        def show_completion(self, preset = None):
-                buf = self.view.get_buffer()
-                bounds = buf.get_selection_bounds()
-                prefix = None
-                
-                if not bounds and not preset:
-                        # When there is no text selected and no preset present, find the
-                        # prefix
-                        (prefix, start, end) = self.get_tab_tag(buf)
-                
-                if not prefix:
-                        # If there is no prefix, than take the insertion point as the end
-                        end = buf.get_iter_at_mark(buf.get_insert())
-                
-                if not preset or len(preset) == 0:
-                        # There is no preset, find all the global snippets and the language
-                        # specific snippets
-                        
-                        nodes = Library().get_snippets(None)
-                        
-                        if self.language_id:
-                                nodes += Library().get_snippets(self.language_id)
-                        
-                        if prefix and len(prefix) == 1 and not prefix.isalnum():
-                                hasnodes = False
-                                
-                                for node in nodes:
-                                        if node['tag'] and node['tag'].startswith(prefix):
-                                                hasnodes = True
-                                                break
-                                
-                                if not hasnodes:
-                                        prefix = None
-                        
-                        complete = SnippetComplete(nodes, prefix, False)        
-                else:
-                        # There is a preset, so show that preset
-                        complete = SnippetComplete(preset, None, True)
-                
-                complete.connect('snippet-activated', self.on_complete_row_activated)
-                
-                rect = self.view.get_iter_location(end)
-                win = self.view.get_window(gtk.TEXT_WINDOW_TEXT)
-                (x, y) = self.view.buffer_to_window_coords( \
-                                gtk.TEXT_WINDOW_TEXT, rect.x + rect.width, rect.y)
-                (xor, yor) = win.get_origin()
-                
-                self.move_completion_window(complete, x + xor, y + yor)                
-                return complete.run()
+                self.view.queue_draw()
 
         def update_snippet_contents(self):
                 self.timeout_update_id = 0
-                
+
                 for placeholder in self.update_placeholders:
                         placeholder.update_contents()
-                
+
                 for placeholder in self.jump_placeholders:
                         self.goto_placeholder(placeholder[0], placeholder[1])
-                
+
                 del self.update_placeholders[:]
                 del self.jump_placeholders[:]
-                
+
                 return False
 
         # Callbacks
@@ -704,16 +706,6 @@ class Document:
                 self.stop()
                 return
 
-        def on_complete_row_activated(self, complete, snippet):
-                buf = self.view.get_buffer()
-                bounds = buf.get_selection_bounds()
-                
-                if bounds:
-                        self.apply_snippet(snippet.data, None, None)
-                else:
-                        (word, start, end) = self.get_tab_tag(buf)
-                        self.apply_snippet(snippet.data, start, end)
-
         def on_buffer_cursor_moved(self, buf):
                 piter = buf.get_iter_at_mark(buf.get_insert())
 
@@ -728,22 +720,19 @@ class Document:
                                 if piter.compare(begin) < 0 or piter.compare(end) > 0:
                                         # Oh no! Remove the snippet this instant!!
                                         self.deactivate_snippet(snippet)
-                
-                current = self.current_placeholder()
-                
-                if current != self.active_placeholder:
-                        if self.active_placeholder:
-                                self.jump_placeholders.append((self.active_placeholder, current))
-                                
-                                if self.timeout_update_id == 0:
-                                        self.timeout_update_id = gobject.timeout_add(0, 
-                                                        self.update_snippet_contents)
 
-                        self.set_active_placeholder(current)
-        
+                current = self.current_placeholder()
+
+                if current != self.active_placeholder:
+                        self.jump_placeholders.append((self.active_placeholder, current))
+
+                        if self.timeout_update_id == 0:
+                                self.timeout_update_id = gobject.timeout_add(0, 
+                                                self.update_snippet_contents)
+
         def on_buffer_changed(self, buf):
                 current = self.current_placeholder()
-                
+
                 if current:
                         if not current in self.update_placeholders:
                                 self.update_placeholders.append(current)
@@ -807,11 +796,6 @@ class Document:
                                 return self.run_snippet()
                         else:
                                 return self.skip_to_previous_placeholder()
-                elif (event.state & gdk.CONTROL_MASK) and \
-                                not (event.state & gdk.MOD1_MASK) and \
-                                not (event.state & gdk.SHIFT_MASK) and \
-                                event.keyval in self.SPACE_KEY_VAL:
-                        return self.show_completion()
                 elif not library.loaded and \
                                 library.valid_accelerator(event.keyval, event.state):
                         library.ensure_files()
@@ -945,4 +929,152 @@ class Document:
                 lst = gtk.target_list_add_uri_targets((), 0)
                 
                 return self.view.drag_dest_find_target(context, lst)
+        
+        def on_completion_hide(self, completion):
+                self.provider.set_proposals(None)
+
+        def on_proposal_activated(self, proposal, piter):
+                buf = self.view.get_buffer()
+                bounds = buf.get_selection_bounds()
+                
+                if bounds:
+                        self.apply_snippet(proposal.snippet(), None, None)
+                else:
+                        (word, start, end) = self.get_tab_tag(buf, piter)
+                        self.apply_snippet(proposal.snippet(), start, end)
+
+                return True
+        
+        def on_default_activated(self, proposal, piter):
+                buf = self.view.get_buffer()
+                bounds = buf.get_selection_bounds()
+
+                if bounds:
+                        buf.begin_user_action()
+                        buf.delete(bounds[0], bounds[1])
+                        buf.insert(bounds[0], proposal.props.label)
+                        buf.end_user_action()
+
+                        return True
+                else:
+                        return False
+
+        def iter_coords(self, piter):
+                rect = self.view.get_iter_location(piter)
+                rect.x, rect.y = self.view.buffer_to_window_coords(gtk.TEXT_WINDOW_TEXT, rect.x, rect.y)
+
+                return rect
+
+        def placeholder_in_area(self, placeholder, area):
+                start = placeholder.begin_iter()
+                end = placeholder.end_iter()
+
+                # Test if start is before bottom, and end is after top
+                start_rect = self.iter_coords(start)
+                end_rect = self.iter_coords(end)
+
+                return start_rect.y <= area.y + area.height and \
+                       end_rect.y + end_rect.height >= area.y
+
+        def draw_placeholder_rect(self, ctx, placeholder, col):
+                start = placeholder.begin_iter()
+                start_rect = self.iter_coords(start)
+                start_line = start.get_line()
+
+                end = placeholder.end_iter()
+                end_rect = self.iter_coords(end)
+                end_line = end.get_line()
+
+                line = start.copy()
+                line.set_line_offset(0)
+                geom = self.view.get_window(gtk.TEXT_WINDOW_TEXT).get_geometry()
+                
+                ctx.translate(0.5, 0.5)
+                
+                while line.get_line() <= end_line:
+                        ypos, height = self.view.get_line_yrange(line)
+                        x_, ypos = self.view.window_to_buffer_coords(gtk.TEXT_WINDOW_TEXT, 0, ypos)
+
+                        if line.get_line() == start_line and line.get_line() == end_line:
+                                # Simply draw a box, both are on the same line
+                                ctx.rectangle(start_rect.x, start_rect.y, end_rect.x - start_rect.x, start_rect.height - 1)
+                                ctx.stroke()
+                        elif line.get_line() == start_line or line.get_line() == end_line:
+                                if line.get_line() == start_line:
+                                        rect = start_rect
+                                else:
+                                        rect = end_rect
+
+                                ctx.move_to(0, rect.y + rect.height - 1)
+                                ctx.rel_line_to(rect.x, 0)
+                                ctx.rel_line_to(0, -rect.height + 1)
+                                ctx.rel_line_to(geom[2], 0)
+                                ctx.stroke()
+
+                        if not line.forward_line():
+                                break
+
+        def draw_placeholder_bar(self, ctx, placeholder, col):
+                start = placeholder.begin_iter()
+                start_rect = self.iter_coords(start)
+                
+                ctx.translate(0.5, 0.5)
+                extend_width = 2.5
+
+                ctx.move_to(start_rect.x - extend_width, start_rect.y)
+                ctx.rel_line_to(extend_width * 2, 0)
+
+                ctx.move_to(start_rect.x, start_rect.y)
+                ctx.rel_line_to(0, start_rect.height - 1)
+                
+                ctx.rel_move_to(-extend_width, 0)
+                ctx.rel_line_to(extend_width * 2, 0)
+                ctx.stroke()
+
+        def from_color(self, col):
+                return [col.red / 0x10000, col.green / 0x10000, col.blue / 0x10000]
+
+        def draw_placeholder(self, ctx, placeholder):
+                if isinstance(placeholder, PlaceholderEnd):
+                        return
+
+                buf = self.view.get_buffer()
+
+                col = self.from_color(self.view.get_style().text[gtk.STATE_INSENSITIVE])
+                ctx.set_source_rgba(col[0], col[1], col[2], 0.5)
+                
+                if placeholder.tabstop > 0:
+                        ctx.set_dash([], 0)
+                else:
+                        ctx.set_dash([2], 0)
+
+                start = placeholder.begin_iter()
+                end = placeholder.end_iter()
+
+                if start.equal(end):
+                        self.draw_placeholder_bar(ctx, placeholder, col)
+                else:
+                        self.draw_placeholder_rect(ctx, placeholder, col)
+
+        def on_expose_event(self, view, event):
+                if event.window != view.get_window(gtk.TEXT_WINDOW_TEXT):
+                        return False
+
+                # Draw something
+                ctx = event.window.cairo_create()
+                ctx.rectangle(event.area)
+                ctx.clip()
+
+                ctx.set_line_width(1.0)
+
+                for placeholder in self.ordered_placeholders:
+                        if not self.placeholder_in_area(placeholder, event.area):
+                                continue
+
+                        ctx.save()
+                        self.draw_placeholder(ctx, placeholder)
+                        ctx.restore()
+
+                return False
+
 # ex:ts=8:et:
